package scripts;

import com.stencyl.graphics.G;
import com.stencyl.graphics.BitmapWrapper;
import com.stencyl.graphics.ScaleMode;

import com.stencyl.behavior.Script;
import com.stencyl.behavior.Script.*;
import com.stencyl.behavior.ActorScript;
import com.stencyl.behavior.SceneScript;
import com.stencyl.behavior.TimedTask;

import com.stencyl.models.Actor;
import com.stencyl.models.GameModel;
import com.stencyl.models.actor.Animation;
import com.stencyl.models.actor.ActorType;
import com.stencyl.models.actor.Collision;
import com.stencyl.models.actor.Group;
import com.stencyl.models.Scene;
import com.stencyl.models.Sound;
import com.stencyl.models.Region;
import com.stencyl.models.Font;
import com.stencyl.models.Joystick;

import com.stencyl.Config;
import com.stencyl.Engine;
import com.stencyl.Input;
import com.stencyl.Key;
import com.stencyl.utils.motion.*;
import com.stencyl.utils.Utils;

import openfl.ui.Mouse;
import openfl.display.Graphics;
import openfl.display.BlendMode;
import openfl.display.BitmapData;
import openfl.display.Bitmap;
import openfl.events.Event;
import openfl.events.KeyboardEvent;
import openfl.events.TouchEvent;
import openfl.net.URLLoader;

import box2D.common.math.B2Vec2;
import box2D.dynamics.B2Body;
import box2D.dynamics.B2Fixture;
import box2D.dynamics.joints.B2Joint;

import com.stencyl.graphics.shaders.BasicShader;
import com.stencyl.graphics.shaders.GrayscaleShader;
import com.stencyl.graphics.shaders.SepiaShader;
import com.stencyl.graphics.shaders.InvertShader;
import com.stencyl.graphics.shaders.GrainShader;
import com.stencyl.graphics.shaders.ExternalShader;
import com.stencyl.graphics.shaders.InlineShader;
import com.stencyl.graphics.shaders.BlurShader;
import com.stencyl.graphics.shaders.SharpenShader;
import com.stencyl.graphics.shaders.ScanlineShader;
import com.stencyl.graphics.shaders.CSBShader;
import com.stencyl.graphics.shaders.HueShader;
import com.stencyl.graphics.shaders.TintShader;
import com.stencyl.graphics.shaders.BloomShader;



class Design_14_14_FollowDragandDrop extends ActorScript
{
	public var _XOffset:Float;
	public var _YOffset:Float;
	public var _Grabbed:Bool;
	public var _OldX:Float;
	public var _OldY:Float;
	public var _DistanceX:Float;
	public var _DistanceY:Float;
	public var _Distance:Float;
	public var _Direction:Float;
	public var _MinimumSpeed:Float;
	public var _Speed:Float;
	public var _ScreenDiagonal:Float;
	public var _KeepMovingonRelease:Bool;
	public var _ReleaseSpeedFactor:Float;
	
	
	public function new(dummy:Int, actor:Actor, dummy2:Engine)
	{
		super(actor);
		nameMap.set("Actor", "actor");
		nameMap.set("X Offset", "_XOffset");
		_XOffset = 0.0;
		nameMap.set("Y Offset", "_YOffset");
		_YOffset = 0.0;
		nameMap.set("Grabbed", "_Grabbed");
		_Grabbed = false;
		nameMap.set("Old X", "_OldX");
		_OldX = 0.0;
		nameMap.set("Old Y", "_OldY");
		_OldY = 0.0;
		nameMap.set("Distance X", "_DistanceX");
		_DistanceX = 0.0;
		nameMap.set("Distance Y", "_DistanceY");
		_DistanceY = 0.0;
		nameMap.set("Distance", "_Distance");
		_Distance = 0.0;
		nameMap.set("Direction", "_Direction");
		_Direction = 0.0;
		nameMap.set("Minimum Speed", "_MinimumSpeed");
		_MinimumSpeed = 5.0;
		nameMap.set("Speed", "_Speed");
		_Speed = 1000.0;
		nameMap.set("Screen Diagonal", "_ScreenDiagonal");
		_ScreenDiagonal = 0.0;
		nameMap.set("Keep Moving on Release", "_KeepMovingonRelease");
		_KeepMovingonRelease = false;
		nameMap.set("Release Speed Factor", "_ReleaseSpeedFactor");
		_ReleaseSpeedFactor = 0.5;
		
	}
	
	override public function init()
	{
		
		/* ======================== When Creating ========================= */
		_ScreenDiagonal = Math.sqrt((Math.pow(getScreenWidth(), 2) + Math.pow(getScreenHeight(), 2)));
		
		/* ======================== When Updating ========================= */
		addWhenUpdatedListener(null, function(elapsedTime:Float, list:Array<Dynamic>):Void
		{
			if(wrapper.enabled)
			{
				if(_Grabbed)
				{
					if(isMouseDown())
					{
						_DistanceX = ((getScreenX() + getMouseX()) - (actor.getXCenter() - _XOffset));
						_DistanceY = ((getScreenY() + getMouseY()) - (actor.getYCenter() - _YOffset));
						_Distance = Math.sqrt((Math.pow(_DistanceX, 2) + Math.pow(_DistanceY, 2)));
						_Direction = Utils.DEG * (Math.atan2(_DistanceY, _DistanceX));
						if((_Distance > 0))
						{
							actor.setVelocity(_Direction, (_MinimumSpeed + (_Speed * (_Distance / _ScreenDiagonal))));
						}
						else
						{
							actor.setVelocity(0, 0);
						}
						actor.setAngularVelocity(Utils.RAD * (0));
					}
					if(isMouseReleased())
					{
						if(_KeepMovingonRelease)
						{
							actor.setXVelocity((actor.getXVelocity() * _ReleaseSpeedFactor));
							actor.setYVelocity((actor.getYVelocity() * _ReleaseSpeedFactor));
						}
						else
						{
							actor.setVelocity(0, 0);
						}
						_Grabbed = false;
					}
				}
				else
				{
					if(actor.isMousePressed())
					{
						_XOffset = (actor.getXCenter() - (getScreenX() + getMouseX()));
						_YOffset = (actor.getYCenter() - (getScreenY() + getMouseY()));
						_Grabbed = true;
					}
				}
			}
		});
		
	}
	
	override public function forwardMessage(msg:String)
	{
		
	}
}